//---------------------------------------------------------------------------
// Message.cpp - displays a user-defined messagebox on the screen and sets
//	the exit code based on the output.  Relies heavily on the flag
//	definitions used by the messagebox and found in winuser.h.  Supports
//	substitution of $...$ with current date/time.
//
// Released to the public domain on March 12, 1998 by
//	Rik Blok <riblok@mail.com>, who takes no responsibility for this
//	program.
//
// Revisions:
// v1.2 November 18, 2000
//	- if icon selected then force window to foreground
//	- set taskbar title to message title
//	- 40% smaller executable (by only including necessary headers)
//
// v1.1 March 10, 1999
//	- allow text to be read from standard input
//
// v1.0	March 12, 1998
//	- initial release
//---------------------------------------------------------------------------
#include <vcl\dstring.h>	// AnsiString
#include <vcl\sysdefs.h>	// TDateTime
#include <vcl\system.hpp>	// ParamCount(), ParamStr()
#include <stdio.h>			// getchar() and EOF
#include <windows.h>		// GetCommandLine(), GetFileTitle(),SetForegroundWindow(), SetWindowText(), MessageBox()
#pragma hdrstop
#define VERSION 1.2	// November 18, 2000
//---------------------------------------------------------------------------
AnsiString ExpandDateTime(AnsiString in)
// Returns in after substituting the current date/time for pairs of $...$
// which contain date/time formatting information.
// Note: remember AnsiString indices start at 1
{
	static TDateTime now = TDateTime::CurrentDateTime();
	char delim = '$';
	AnsiString out = "";
    int pos;
	while ((pos=in.Pos(delim))>0) {	// while found opening delim at index pos
		out += in.SubString(1,pos-1);
        in = in.SubString(pos+1,in.Length()-pos);
    	if ((pos=in.Pos(delim))>1) {	// if found closing delim at index pos>1
        	out += now.FormatString(in.SubString(1,pos-1));
            in = in.SubString(pos+1,in.Length()-pos);
        } else if (pos==1) {			// if found closing delim at index 1
        	out += in[1];				// then copy delimiter to output
            in = in.SubString(2,in.Length()-1);
        } else {						// closing delim not found
            out += in;					// so just copy remainder
			in = "";
        }
	}
    out += in;
	return out;
}
//---------------------------------------------------------------------------
AnsiString GetProgTitle(void)
// get executable name sans path and extension
{
    char buffer[256];
	AnsiString Program = AnsiString(GetCommandLine());
	// strip path info
	GetFileTitle(Program.c_str(),buffer,256);
	Program = buffer;
	// strip extension
	Program = Program.SubString(1,Program.LastDelimiter(".")-1);
	return Program;
}
//---------------------------------------------------------------------------
AnsiString ReadStdIn(void)
// read input from stdin until end-of-file (or ctrl-z?)
{
	AnsiString buffer = "";
	int c;
	while ((c = getchar()) != EOF)
		buffer += (char) c;
	return buffer;
}
//---------------------------------------------------------------------------
WINAPI WinMain(HINSTANCE handle, HINSTANCE, LPSTR, int)
{
	AnsiString Program = GetProgTitle();
	// message (default to help)
	AnsiString Text;
	// allow input from stdin [v1.1]
	if (ParamStr(1) == "-")	Text = ReadStdIn();
	else					Text = ParamStr(1);
	Text = ExpandDateTime(Text);
	if (Text.IsEmpty()) {
		// help message
		Text = "Usage: " + Program + " \"message\" [\"title\"] [buttons] [icon]\n\n"
			+ "where buttons is one of\n"
			+ "\tOk (default),\n"
			+ "\tOkCancel,\n"
			+ "\tAbortRetryIgnore,\n"
			+ "\tYesNoCancel,\n"
			+ "\tYesNo, or\n"
			+ "\tRetryCancel\n\n"
			+ "and icon is one of\n"
			+ "\tNone (default),\n"
			+ "\tError,\n"
			+ "\tQuestion,\n"
			+ "\tWarning, or\n"
			+ "\tInformation.\n\n"
			+ "Returns an integer (0..7) indicating\n"
			+ "\t0 = error creating message box\n"
			+ "\t1 = user pressed Ok\n"
			+ "\t2 = user pressed Cancel\n"
			+ "\t3 = user pressed Abort\n"
			+ "\t4 = user pressed Retry\n"
			+ "\t5 = user pressed Ignore\n"
			+ "\t6 = user pressed Yes\n"
			+ "\t7 = user pressed No";
	}

	// title (default to program name)
	AnsiString Title;
	if (ParamCount() < 2)	Title = Program;
	else					Title = ExpandDateTime(ParamStr(2));

	// button(s) (default to OK)
	const unsigned short ButtonCount = 6;
	AnsiString Buttons[ButtonCount] = {"ok", "okcancel", "abortretryignore",
		"yesnocancel", "yesno", "retrycancel"};
	unsigned short bFlag;
	for (bFlag=ButtonCount-1; bFlag>0; bFlag--)
		if (ParamStr(3).LowerCase() == Buttons[bFlag])	break;

	// icon (default to none)
	const unsigned short IconCount = 5;
	AnsiString Icons[IconCount] = {"none","error", "question","warning",
		"information"};
	unsigned short iFlag;
	for (iFlag=IconCount-1; iFlag>0; iFlag--)
		if (ParamStr(4).LowerCase() == Icons[iFlag])	break;

	// if icon then force window to foreground
	if (iFlag)	SetForegroundWindow(handle);

	// display and return result code
	SetWindowText(handle,Title.c_str());
	#pragma warn -sig		// disable warning "Conversion may lose signific..."
	return MessageBox(NULL, Text.c_str(), Title.c_str(),
		bFlag+0x10*iFlag);	// reset warning
	#pragma warn .sig
}
//---------------------------------------------------------------------------
